#!/usr/bin/perl
# check_jboss.pl - Check thread and memory statistics for Jboss
# Copyright (C) 2011  DataWolf
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use strict;
use LWP::Simple;
use Getopt::Std;

our %opts = ('H' => 'localhost', 'P' => '8080', 'p' => '8080', 't' => '80,50', 'm' => '80,50', 'a' => '8009', 'h' => '8080,8443');
our $exit = 0;
our $status;
our $DEBUG = 0;
my $httpEndpoint;
my $threadCrit;
my $threadWarn;
my $memCrit;
my $memWarn;
my @mem;
my @thread;
my @ajpPorts = (8009);
my @httpPorts = (8080,8443);

$Getopt::Std::STANDARD_HELP_VERSION = 1;
getopts('H:P:a:h:t:m:d', \%opts);

$httpEndpoint = "http://" . $opts{'H'} . ":" . $opts{'P'} . '/status';
$DEBUG = $opts{'d'} != 0;

if ($DEBUG) {
	print "End Point:\t".$httpEndpoint."\n";
	printf("Critical:\t%03d%%\nWarning:\t%03d%%\n",$opts{'c'},$opts{'w'});
}

#####  Get thread thresholds  #####
@thread = split(/,/, $opts{'t'});
$threadCrit = $thread[0];
$threadWarn = $thread[1];

#####  Get memory thresholds  #####
@mem = split(/,/, $opts{'m'});
$memCrit = $mem[0];
$memWarn = $mem[1];

#####  Get our ports  #####
@ajpPorts = split(/,/, $opts{'a'});
@httpPorts = split(/,/, $opts{'h'});

#####  Grab the status page  #####
$status = get($httpEndpoint) or unable_to_load("$!");

#####  Check how much memory is available  #####
check_mem($memCrit, $memWarn);

#####  Check our AJP ports  #####
foreach(@ajpPorts) {
	check_ajp($_, $threadCrit, $threadWarn);
}

#####  Check the HTTP ports  #####
foreach(@httpPorts) {
	check_http($_, $threadCrit, $threadWarn);
}

#####  Exit with the return code  #####
exit($exit);

#####  The subroutine for memory checking  #####
sub check_mem {
	my $crit = shift;
	my $warn = shift;
	my $check = $status;
	my $max = 0;
	my $cur = 0;
	my $pct = 0;
	
	if($check =~ /<h1>JVM<\/h1>/)
	{
		$check =~ s#^.*<h1>JVM</h1><p> ([^<]+)<.*#\1#;
		
		if($DEBUG) {
			print "\nStatus:\t\t$check\n";
		}
		
		$max = $check;
		$cur = $check;
		
		$max =~ s/.*Max memory: (\d+(\.\d+)) MB.*/\1/;
		$cur =~ s/.*Free memory: (\d+(\.\d+)) MB.*/\1/;
		
		if($DEBUG) {
			printf("\nMax:\t\t%.2f\nCurrent:\t%.2f\n",$max,$cur);
		}
		
		$pct = (($max - $cur) / $max) * 100;
		printf("Memory: %.2f/%.2f (%2.0f%%)", ($max - $cur), $max, $pct);
		
		if($pct >= $warn && $pct < $crit) {
			print " WARNING";
			if($exit < 1) {
				$exit = 1;
			}
		}
		
		if($pct >= $crit) {
			print " CRITICAL";
			if($exit < 2)
			{
				$exit = 2;
			}
		}
		
		if($pct < $warn && $pct < $crit) {
			print " OK";
		}
		
		print "; ";
	} else {
		printf("Unable to get memory!");
		$exit = 2;
	}
}

#####  The subroutine for AJP thread checking  #####
sub check_ajp {
	my $port = shift;
	my $crit = shift;
	my $warn = shift;
	my $maxThreads = 0;
	my $currentBusy = 0;
	my $threadPct = 0;
	my $check = $status;
	
	if($check =~ /<h1>(jk|(ajp-\d+\.\d+\.\d+\.\d+))-$port<\/h1>/)
	{
		$check =~ s#^.*<h1>(jk|(ajp-\d+\.\d+\.\d+\.\d+))-$port</h1><p> ([^<]+)<br>.*$#\3#;

		if($DEBUG) {
			print "\nStatus:\t\t$check\n";
		}
		
		$maxThreads = $check;
		$currentBusy = $check;
		$maxThreads =~ s/^.*Max threads: (\d+) .*$/\1/;
		$currentBusy =~ s/^.*Current thread busy: (\d+)$/\1/;
		
		if($DEBUG) {
			printf("Current Busy:\t%03d\nMax threads:\t%03d\n", $currentBusy, $maxThreads);
		}
		
		$threadPct = ($currentBusy / $maxThreads) * 100;
		
		printf("ajp(%d): %d/%d (%.0f%%) threads busy", $port, $currentBusy, $maxThreads, $threadPct);
		if($warn <= $threadPct && $threadPct < $crit) {
			print " WARNING ";
			if($exit < 1) {
				$exit = 1;
			}
		}
		
		if($crit <= $threadPct) {
			print " CRITICAL";
			if($exit < 2) {
				$exit = 2;
			}
		}
		
		if($threadPct < $warn && $threadPct < $crit) {
			print " OK";
		}
		
		print "; ";
	} else {
		if($DEBUG) {
			printf("\najp(%d): not running;\n", $port);
		}
	}
}

#####  The subroutine for HTTP thread checking  #####
sub check_http {
	my $port = shift;
	my $crit = shift;
	my $warn = shift;
	my $maxThreads = 0;
	my $currentBusy = 0;
	my $threadPct = 0;
	my $check = $status;
	
	if($check =~ /<h1>http-\d+\.\d+\.\d+\.\d+-$port<\/h1>/) {
		$check =~ s#^.*<h1>http-\d+\.\d+\.\d+\.\d+-$port</h1><p> ([^<]+)<br>.*$#\1#;

		if($DEBUG) {
			print "\nStatus:\t\t$check\n";
		}

		$maxThreads = $check;
		$currentBusy = $check;
		$maxThreads =~ s/^.*Max threads: (\d+) .*$/\1/;
		$currentBusy =~ s/^.*Current thread busy: (\d+)$/\1/;
		
		if($DEBUG) {
			printf("Current Busy:\t%03d\nMax threads:\t%03d\n", $currentBusy, $maxThreads);
		}
		
		$threadPct = ($currentBusy / $maxThreads) * 100;
		
		printf("http(%d): %d/%d (%.0f%%) threads busy", $port, $currentBusy, $maxThreads, $threadPct);
		if($warn <= $threadPct && $threadPct < $crit) {
			print " WARNING ";
			if($exit < 1) {
				$exit = 1;
			}
		}
		
		if($crit <= $threadPct) {
			print " CRITICAL";
			if($exit < 2) {
				$exit = 2;
			}
		}
		
		if($threadPct < $warn && $threadPct < $crit) {
			print " OK";
		}
		
		print "; ";
	} else {
		if($DEBUG) {
			printf("\nhttp(%d): not running;\n", $port);
		}
	}
}

#####  Subroutine for the version  #####
sub VERSION_MESSAGE {
	my $fh = shift;
	print $fh "$0 - Jboss Status Check for Nagios\nVersion 1.0\n";
	print $fh "Copyright (C) 2011  DataWolf\n";
	print $fh "\n";
	print $fh "This program is free software; you can redistribute it and/or\n";
	print $fh "modify it under the terms of the GNU General Public License\n";
	print $fh "as published by the Free Software Foundation; either version 2\n";
	print $fh "of the License, or (at your option) any later version.\n";
	print $fh "\n";
	print $fh "This program is distributed in the hope that it will be useful,\n";
	print $fh "but WITHOUT ANY WARRANTY; without even the implied warranty of\n";
	print $fh "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n";
	print $fh "GNU General Public License for more details.\n";
	print $fh "\n";
	print $fh "You should have received a copy of the GNU General Public License\n";
	print $fh "along with this program; if not, write to the Free Software\n";
	print $fh "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.\n";
}

#####  Subroutine for the help message  #####
sub HELP_MESSAGE {
	my $fh = shift;
	
	          #         1         2         3         4         5         6         7         8
	          #12345678901234567890123456789012345678901234567890123456789012345678901234567890
	print $fh "\nUsage: check_jboss_status.pl [-H <host>] [-P <port>] [-t <crit%,warn%>]\n";
	print $fh "                      [-m <crit%,warn%>] [-a <port1,port2,...,portn>]\n";
	print $fh "                      [-h <port1,port2,...,portn>]\n";
	print $fh "    -H <host>                   The host to connect to (default: ".$opts{'H'}.")\n";
	print $fh "    -P <port>                   The port to connect to (default: ".$opts{'P'}.")\n";
	print $fh "    -a <port1,port2,...,portn>  The AJP ports to check for (default: ".$opts{'a'}.")\n";
	print $fh "    -h <port1,port2,...,portn>  The HTTP ports to check for (default: ".$opts{'h'}.")\n";
	print $fh "    -t <crit%,warn%>            The percentage of threads busy for warning\n";
	print $fh "                                (default: ".$opts{'t'}.")\n";
	print $fh "    -m <crit%,warn%>            The percentage of memory used for warning\n";
	print $fh "                                (default: ".$opts{'m'}.")\n";
	print $fh "\n";
	print $fh "Example:\n";
	print $fh "    check_jboss_status.pl -H jboss.example.com -P 8080 -t 90,75 -m 75,50 -a 8009 -h\n";
	print $fh "                   8080,8443\n";
	print $fh "This will check the host \"jboss.example.com\" on port \"8080\" and\n";
	print $fh "issue a warning at 75% thread busy or a critical alert at 90% thread\n";
	print $fh "usage, or a warning at 50% or critical at 75% JVM memory usage checking.\n";
	print $fh "for AJP port 8009 and HTTP ports 8080 and 8443.\n";
}

sub unable_to_load {
	my $ex = shift;
	
	print "Unable to load status: $ex";
	exit(2);
}

